%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 3D Evaluation Code for:
%
% Are Cars Just 3D Boxes? - Jointly Estimating the 3D Shape of Multiple Objects
% M Zeeshan Zia, Michael Stark, Konrad Schindler
% IEEE Conference on Computer Vision and Pattern Recognition (CVPR), 2014
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
close all;

load Zia_CVPR14_imagelist.mat;   % List of test images

load KITTI_CVPR14_labelled_260ims.mat; % Ground truth annotation of these cars 
                                       % (2D part locations, part-level
                                       % occlusion labels, 2D bounding
                                       % boxes which are projections of ground truth 
                                       % 3D bounding boxes**) 
                                       
% **Instead of using ground truth 2D bounding boxes for estimating 2D
% bounding box-level match, we use projections of the 3D bounding boxes.
% This is because the 2D bounding boxes provided with the dataset are 
% truncated by the image boundaries, whereas our detector provides 
% detections which may even fall partially outside the image boundaries. 
% Using projections of 3D bounding boxes gives us 2D bounding box whose 
% coordinates are not limited to the image region. Ofcourse on the downside
% calculating a "ground truth" 2D bounding box using this approach always 
% over-estimates the ground truth 2D bounding box - but we believe this 
% to be a superior approach compared to using truncated 2D bounding boxes that 
% vastly under-estimate the 2D extent of objects truncated by image
% boundary.
                                       

load zia_cvpr14_FG+GP+DO+SO.mat; % Zia CVPR'14 detections (FG+GP+DO+SO)

% Initialize variables 
total_cars = 0; % counts no. of cars with correct 2D bbx overlap (after running the full pipeline)
correct_cars_1m = 0; % counts no. of cars within 1m distance from ground truth
correct_cars_2m = 0; 

correct_vp_cnt_less10 = 0;  % no. of cars with viewpoint estimated within 10 degrees
avg_vp_error = 0;
avg_vp_error_list = [];

% Main loop over all the test images
for imgind=1:260
    if (length(detected_objects{imgind}) > 0)
        
        gt_already_considered = zeros(length(KITTI_testimage(imgind).car),1);
        
        % Ground truth annotations downloaded from http://www.cvlibs.net/download.php?file=data_object_label_2.zip
        objects = readLabels('/BS/starkprojekt/work/zeeshan-michael/KITTI_subset/training/label_2',images_to_include(imgind));

        for p=1:int16(length(detected_objects{imgind}))

            x1 = detected_objects{imgind}(p).x1;
            x2 = detected_objects{imgind}(p).x2;
            y1 = detected_objects{imgind}(p).y1;
            y2 = detected_objects{imgind}(p).y2;

            rect_hyp.x1 = x1;
            rect_hyp.y1 = y1;
            rect_hyp.x2 = x2;
            rect_hyp.y2 = y1;
            rect_hyp.x3 = x2;
            rect_hyp.y3 = y2;
            rect_hyp.x4 = x1;
            rect_hyp.y4 = y2;    


            % For all ground truth (gt) objects in this test image.
            % We search one ground truth object for each detected
            % object in a greedy way - based on 2D bounding box overlap.
            best_gt_car_overlap = 0;
            best_gt_car_id = 0;
            
            for obj_idx_KITTI=1:length(KITTI_testimage(imgind).car)%numel(objects)
                % KITTI_testimage contains 2D annotations made using the
                % tool <<label_CVPR14_dataset.m>>.
                x1 = KITTI_testimage(imgind).car(obj_idx_KITTI).x1; % Ground truth 2D bounding box coordinates.
                x2 = KITTI_testimage(imgind).car(obj_idx_KITTI).x2; % Actually 2D projection of the ground truth 3D bounding box.
                y1 = KITTI_testimage(imgind).car(obj_idx_KITTI).y1; % The coordinates maybe negative/ outside image boundaries, 
                y2 = KITTI_testimage(imgind).car(obj_idx_KITTI).y2; % specially for the truncated objects.
                
                rect_gt.x1 = x1;
                rect_gt.y1 = y1;
                rect_gt.x2 = x2;
                rect_gt.y2 = y1;
                rect_gt.x3 = x2;
                rect_gt.y3 = y2;
                rect_gt.x4 = x1;
                rect_gt.y4 = y2;        

                rat = get_iou(rect_gt,rect_hyp);    % returns the ratio of intersection area to union area
                
                if (rat > best_gt_car_overlap && rat > 0.5 && gt_already_considered(obj_idx_KITTI) ==0)
                    best_gt_car_overlap = rat;
                    best_gt_car_id = obj_idx_KITTI;
                end
            end
            
            if (best_gt_car_id ~= 0) % found a corresponding 2D detection
                
                obj_idx_KITTI = best_gt_car_id;
                rat = best_gt_car_overlap;
                obj_idx = KITTI_testimage(imgind).car(obj_idx_KITTI).objectid;  
                gt_already_considered(obj_idx_KITTI) = 1;
                
                % Full-dataset evaluation (uncomment next line)
                if (rat > 0.5 && compare_vp_biased(KITTI_testimage(imgind).car(obj_idx_KITTI).VPanglediscrete,45*(closest_viewpoint(180+(detected_objects{imgind}(p).alpha * 180/pi))-1))==1 && (KITTI_testimage(imgind).car(obj_idx_KITTI).y2-KITTI_testimage(imgind).car(obj_idx_KITTI).y1) >= 50 && (length(KITTI_testimage(imgind).car(obj_idx_KITTI).occluded) - sum(KITTI_testimage(imgind).car(obj_idx_KITTI).occluded)) >= 4 && sum(KITTI_testimage(imgind).car(obj_idx_KITTI).type(1:3) =='Car')==3)
                
                % Evaluation only on occluded cars (uncomment next line)
%               if (rat > 0.5 && compare_vp_biased(KITTI_testimage(imgind).car(obj_idx_KITTI).VPanglediscrete,45*(closest_viewpoint(ICCV_imdet.hyp(p).particle_pack(8) - 90)-1))==1 && (KITTI_testimage(imgind).car(obj_idx_KITTI).y2-KITTI_testimage(imgind).car(obj_idx_KITTI).y1) >= 50 && (length(KITTI_testimage(imgind).car(obj_idx_KITTI).occluded) - sum(KITTI_testimage(imgind).car(obj_idx_KITTI).occluded)) >= 4 && sum(KITTI_testimage(imgind).car(obj_idx_KITTI).type(1:3) =='Car')==3 && sum(KITTI_testimage(imgind).car(obj_idx_KITTI).occluded)>0)

                    total_cars = total_cars+1;
                    
                    % FIND 3D DISTANCE BETWEEN ESTIMATE AND GROUND TRUTH
                    % (AS SEEN FROM A BIRD'S EYE VIEW)
                    distance_bet_cars = sqrt((objects(obj_idx).t(1)-detected_objects{imgind}(p).t(1))^2+(objects(obj_idx).t(2)-detected_objects{imgind}(p).t(2))^2 + (objects(obj_idx).t(3)-detected_objects{imgind}(p).t(3))^2);

                    if (distance_bet_cars < 1)
                        correct_cars_1m = correct_cars_1m+1;
                    end
                    if (distance_bet_cars < 2)
                        correct_cars_2m = correct_cars_2m+1;
                    end                
                    
%                   EVALUATE ESTIMATED VIEWPOINTS
                    dist_tmp_vp = [abs(detected_objects{imgind}(p).ry*180/pi - objects(obj_idx).ry*180/pi),360 - abs(detected_objects{imgind}(p).ry*180/pi - objects(obj_idx).ry*180/pi)];
                    if (min(dist_tmp_vp) <= 10)
                        correct_vp_cnt_less10 = correct_vp_cnt_less10 + 1;
                    end
                    
                    avg_vp_error = avg_vp_error+ min(dist_tmp_vp);
                    avg_vp_error_list = [avg_vp_error_list,min(dist_tmp_vp)];
                    

                 end
             end
         end
    end
end

fprintf('*********************************************************\n');
fprintf('Total Number of 2D cars detected (after the full pipeline): %d\n',total_cars);
fprintf('No. of cars correctly localized within 1m : %d\n',correct_cars_1m);
fprintf('No. of cars correctly localized within 2m : %d\n',correct_cars_2m);
fprintf('No. of cars for which viewpoint estimation is within 10 degrees : %d\n',correct_vp_cnt_less10);
fprintf('Average viewpoint estimation error : %f degrees\n',avg_vp_error/total_cars);
fprintf('Median VP error = %f degrees\n***********************************************\n',median(avg_vp_error_list));
fprintf('Total Number of 2D cars detected in the pre-detection stage : 517 (full dataset), 234 (occluded cars only)\n');


